import { Getter, Predicate } from '../types';
/**
 * Functions to be used in {@link Array.prototype.filter} as a callback.
 *
 * @example
 * [1, 2, 2].filter(toBe.unique)
 */
export declare namespace toBe {
    /**
     * Function to be used in {@link Array.prototype.filter} as a callback.
     * Filters out items that are not present ({@code undefined} and {@code null}) in array
     *
     * @returns {boolean}
     */
    function present<T>(value: T): boolean;
    /**
     * Function to be used in {@link Array.prototype.filter} as a callback.
     * Filters out items that are empty
     *
     * @returns {boolean}
     */
    function notEmpty<T>(value: T): boolean;
    /**
     * Function to be used in {@link Array.prototype.filter} as a callback.
     * Filters out items, that are not equal to provided item in parameters.
     * Objects are compared to be deep equal.
     *
     * @returns {function}
     */
    function equal<T>(valueToMatch: T): (value: T) => boolean;
    /**
     * Function to be used in {@link Array.prototype.filter} as a callback.
     * Filters out items, that are equal to provided item in parameters.
     * Objects are compared to be deep equal.
     *
     * @returns {function}
     */
    function notEqual<T>(valueToMatch: T): (value: T) => boolean;
    /**
     * Function to be used in {@link Array.prototype.filter} as a callback.
     * Determines uniqueness of an objects in array. Be aware that if value
     * is not primitive, deep object equality will be checked to determine
     * uniqueness.
     *
     * @returns () => {boolean}
     *
     * @example
     * [1, 2, 2].filter(toBe.unique())
     * // [1, 2]
     * [{a: 1}, {a: 1}, {a: 2}].filter(toBe.unique())
     * // [{a: 1}, {a: 2}]
     */
    function unique(): <T>(value: T) => boolean;
    /**
     * Function to be used in {@link Array.prototype.filter} as a callback.
     * Determines uniqueness by value from callback. This value must be
     * comparable with strict equals
     * @param { Function } getValue     callback to resolve comparable value
     * @example
     * [
     *  { title: 'Predator', genre: 'sci-fi' },
     *  { title: 'Predator 2', genre: 'sci-fi' },
     *  { title: 'Alien vs Predator', genre: 'sci-fi' },
     *  { title: 'Tom & Jerry', genre: 'cartoon' },
     * ]
     *   .filter(toBe.uniqueBy(movie => movie.genre))
     * // [
     * //  { title: 'Predator', genre: 'sci-fi' },
     * //  { title: 'Tom & Jerry', genre: 'cartoon' }
     * // ]
     */
    function uniqueBy<T, R>(getValue: Getter<T, R>): (value: T, index: number, arr: T[]) => boolean;
    /**
     * Function to be used in {@link Array} filter function as a callback.
     * Determines uniqueness by value resolved by objects key. This value must be
     * comparable with strict equals
     *
     * @param { string } key    objects key to resolve comparable value
     * @example
     * [
     *  { title: 'Predator', genre: 'sci-fi' },
     *  { title: 'Predator 2', genre: 'sci-fi'},
     *  { title: 'Alien vs Predator', genre: 'sci-fi' },
     *  { title: 'Tom & Jerry', genre: 'cartoon' },
     * ]
     *   .filter(toBe.uniqueBy('genre'))
     * // [
     * //  { title: 'Predator', genre: 'sci-fi' },
     * //  { title: 'Tom & Jerry', genre: 'cartoon' }
     * // ]
     */
    function uniqueBy<T, K extends keyof T>(key: K): (value: T, index: number, arr: T[]) => boolean;
    /**
     * Function to be used in {@link Array} filter function as a callback.
     * It will pass items from array, while predicate matches. When predicate
     * returns {@code false} none of the items will pass.
     *
     * @param {function} predicate callback function that returns boolean
     * @example
     * import {toBe} from 'declarative-js'
     * import takeWhile = toBe.takeWhile
     *
     * [
     *  { title: 'Predator', genre: 'sci-fi' },
     *  { title: 'Predator 2', genre: 'sci-fi'},
     *  { title: 'Tom & Jerry', genre: 'cartoon' },
     *  { title: 'Alien vs Predator', genre: 'sci-fi' },
     * ]
     *   .filter(takeWhile(film => film.genre === 'sci-fi'))
     * // =>
     * // [
     * //  { title: 'Predator', genre: 'sci-fi' },
     * //  { title: 'Predator 2', genre: 'sci-fi' }
     * // ]
     */
    function takeWhile<T>(predicate: Predicate<T>): (value: T, index: number) => boolean;
}
